@@ -1,12 +1,12 @@
 /*
-	wsseapi-lite.c
+        wsseapi-lite.c
 
-	WS-Security, lite version (time stamp and user name token only).
+        WS-Security, lite version (time stamp and user name token only).
 
 gSOAP XML Web services tools
 Copyright (C) 2000-2015, Robert van Engelen, Genivia Inc., All Rights Reserved.
 This part of the software is released under one of the following licenses:
-GPL, the gSOAP public license, or Genivia's license for commercial use.
+GPL or the gSOAP public license.
 --------------------------------------------------------------------------------
 gSOAP public license.
 
@@ -82,6 +82,7 @@ The wsse lite API is located in:
 
 You will also need:
 
+- `gsoap/custom/struct_timeval.c` compile and link this file (C and C++).
 - compile all sources with `-DWITH_OPENSSL` to enable HTTPS.
 - if you have zlib installed, compile all sources also with `-DWITH_GZIP`.
 - link with `-lssl -lcrypto -lz -gsoapssl++` (or `-lgsoapssl` for C, or compile `stdsoap2.cpp` for C++ and `stdsoap2.c` for C).
@@ -147,7 +148,8 @@ To obtain the actor or role value (e.g. after receiving a message), use:
 @code
     _wsse__Security *security = soap_wsse_Security(soap);
     if (security)
-    { ... = security->SOAP_ENV__actor; // SOAP 1.1
+    {
+      ... = security->SOAP_ENV__actor; // SOAP 1.1
       ... = security->SOAP_ENV__role;  // SOAP 1.2
 @endcode
 
@@ -189,13 +191,15 @@ Web service operation), use:
 
 @code
     int ns__myMethod(struct soap *soap, ...)
-    { const char *username = soap_wsse_get_Username(soap);
+    {
+      const char *username = soap_wsse_get_Username(soap);
       const char *password;
       if (!username)
         return soap->error; // no username: return FailedAuthentication (from soap_wsse_get_Username)
       password = ...; // lookup password of username
       if (soap_wsse_verify_Password(soap, password))
-      {	int err = soap->error;
+      {
+        int err = soap->error;
         soap_wsse_delete_Security(soap); // remove old security headers
         return err; // password verification failed: return FailedAuthentication
       }
@@ -299,9 +303,15 @@ The server uses the following:
     if (!soap_valid_socket(m = soap_bind(soap, NULL, port, 100))
       ... // error
     for (;;)
-    { if (!soap_valid_socket(s = soap_accept(soap)))
+    {
+      if (!soap_valid_socket(s = soap_accept(soap)))
         ... // error
-      THREAD_CREATE(&tid, (void*(*)(void*))&process_request, soap_copy(soap));
+      else
+      {
+        struct soap *tsoap = soap_copy(soap);
+        while (THREAD_CREATE(&tid, (void*(*)(void*))&process_request, (void*)tsoap))
+          sleep(1);
+      }
     }
     soap_destroy(soap);
     soap_end(soap);
@@ -314,7 +324,8 @@ process the request (on a copy of the soap context struct):
 
 @code
   void *process_request(struct soap *soap)
-  { if (soap_ssl_accept(soap)
+  {
+    if (soap_ssl_accept(soap)
      || soap_serve(soap))
       ... // error
     soap_destroy(soap);
@@ -334,13 +345,15 @@ follows:
 
 @code
   struct CRYPTO_dynlock_value
-  { MUTEX_TYPE mutex;
+  {
+    MUTEX_TYPE mutex;
   };
 
   static MUTEX_TYPE *mutex_buf;
 
   static struct CRYPTO_dynlock_value *dyn_create_function(const char *file, int line)
-  { struct CRYPTO_dynlock_value *value;
+  {
+    struct CRYPTO_dynlock_value *value;
     value = (struct CRYPTO_dynlock_value*)malloc(sizeof(struct CRYPTO_dynlock_value));
     if (value)
       MUTEX_SETUP(value->mutex);
@@ -348,30 +361,35 @@ follows:
   }
 
   static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *l, const char *file, int line)
-  { if (mode & CRYPTO_LOCK)
+  {
+    if (mode & CRYPTO_LOCK)
       MUTEX_LOCK(l->mutex);
     else
       MUTEX_UNLOCK(l->mutex);
   }
 
   static void dyn_destroy_function(struct CRYPTO_dynlock_value *l, const char *file, int line)
-  { MUTEX_CLEANUP(l->mutex);
+  {
+    MUTEX_CLEANUP(l->mutex);
     free(l);
   }
 
   void locking_function(int mode, int n, const char *file, int line)
-  { if (mode & CRYPTO_LOCK)
+  {
+    if (mode & CRYPTO_LOCK)
       MUTEX_LOCK(mutex_buf[n]);
     else
       MUTEX_UNLOCK(mutex_buf[n]);
   }
 
   unsigned long id_function()
-  { return (unsigned long)THREAD_ID;
+  {
+    return (unsigned long)THREAD_ID;
   }
 
   int CRYPTO_thread_setup()
-  { int i;
+  {
+    int i;
     mutex_buf = (MUTEX_TYPE*)malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
     if (!mutex_buf)
       return SOAP_EOM;
@@ -386,7 +404,8 @@ follows:
   }
 
   void CRYPTO_thread_cleanup()
-  { int i;
+  {
+    int i;
     if (!mutex_buf)
       return;
     CRYPTO_set_id_callback(NULL);
@@ -421,7 +440,7 @@ extern "C" {
 
 /******************************************************************************\
  *
- * Common URIs
+ *      Common URIs
  *
 \******************************************************************************/
 
@@ -429,7 +448,7 @@ const char *wsse_PasswordTextURI = "http://docs.oasis-open.org/wss/2004/01/oasis
 
 /******************************************************************************\
  *
- * wsse:Security header element
+ *      wsse:Security header element
  *
 \******************************************************************************/
 
@@ -439,14 +458,18 @@ const char *wsse_PasswordTextURI = "http://docs.oasis-open.org/wss/2004/01/oasis
 @param soap context
 @return _wsse__Security object
 */
-struct _wsse__Security*
+SOAP_FMAC1
+struct _wsse__Security *
+SOAP_FMAC2
 soap_wsse_add_Security(struct soap *soap)
-{ DBGFUN("soap_wsse_add_Security");
+{
+  DBGFUN("soap_wsse_add_Security");
   /* if we don't have a SOAP Header, create one */
   soap_header(soap);
   /* if we don't have a wsse:Security element in the SOAP Header, create one */
   if (!soap->header->wsse__Security)
-  { soap->header->wsse__Security = (_wsse__Security*)soap_malloc(soap, sizeof(_wsse__Security));
+  {
+    soap->header->wsse__Security = (_wsse__Security*)soap_malloc(soap, sizeof(_wsse__Security));
     if (!soap->header->wsse__Security)
       return NULL;
     soap_default__wsse__Security(soap, soap->header->wsse__Security);
@@ -454,6 +477,8 @@ soap_wsse_add_Security(struct soap *soap)
   return soap->header->wsse__Security;
 }
 
+/******************************************************************************/
+
 /**
 @fn _wsse__Security* soap_wsse_add_Security_actor(struct soap *soap, const char *actor)
 @brief Adds Security header element with actor or role attribute.
@@ -461,9 +486,12 @@ soap_wsse_add_Security(struct soap *soap)
 @param actor string
 @return _wsse__Security object
 */
-struct _wsse__Security*
+SOAP_FMAC1
+struct _wsse__Security *
+SOAP_FMAC2
 soap_wsse_add_Security_actor(struct soap *soap, const char *actor)
-{ _wsse__Security *security = soap_wsse_add_Security(soap);
+{
+  _wsse__Security *security = soap_wsse_add_Security(soap);
   DBGFUN1("soap_wsse_add_Security_actor", "actor=%s", actor);
   if (soap->namespaces && !strcmp(soap->namespaces[0].ns, "http://schemas.xmlsoap.org/soap/envelope/"))
     security->SOAP_ENV__actor = soap_strdup(soap, actor);
@@ -472,34 +500,44 @@ soap_wsse_add_Security_actor(struct soap *soap, const char *actor)
   return security;
 }
 
+/******************************************************************************/
+
 /**
 @fn void soap_wsse_delete_Security(struct soap *soap)
 @brief Deletes Security header element.
 @param soap context
 */
+SOAP_FMAC1
 void
+SOAP_FMAC2
 soap_wsse_delete_Security(struct soap *soap)
-{ DBGFUN("soap_wsse_delete_Security");
+{
+  DBGFUN("soap_wsse_delete_Security");
   if (soap->header)
     soap->header->wsse__Security = NULL;
 }
 
+/******************************************************************************/
+
 /**
 @fn _wsse__Security* soap_wsse_Security(struct soap *soap)
 @brief Returns Security header element if present.
 @param soap context
 @return _wsse__Security object or NULL
 */
-struct _wsse__Security*
+SOAP_FMAC1
+struct _wsse__Security *
+SOAP_FMAC2
 soap_wsse_Security(struct soap *soap)
-{ if (soap->header)
+{
+  if (soap->header)
     return soap->header->wsse__Security;
   return NULL;
 }
 
 /******************************************************************************\
  *
- * wsse:Security/wsu:Timestamp header element
+ *      wsse:Security/wsu:Timestamp header element
  *
 \******************************************************************************/
 
@@ -511,16 +549,20 @@ soap_wsse_Security(struct soap *soap)
 @param[in] lifetime expressed in time_t units, or 0 for no expiration
 @return SOAP_OK
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_add_Timestamp(struct soap *soap, const char *id, time_t lifetime)
-{ _wsse__Security *security = soap_wsse_add_Security(soap);
+{
+  _wsse__Security *security = soap_wsse_add_Security(soap);
   time_t now = time(NULL);
   char *created = soap_strdup(soap, soap_dateTime2s(soap, now));
   char *expired = lifetime ? soap_strdup(soap, soap_dateTime2s(soap, now + lifetime)) : NULL;
   DBGFUN1("soap_wsse_add_Timestamp", "id=%s", id?id:"");
   /* allocate a Timestamp if we don't have one already */
   if (!security->wsu__Timestamp)
-  { security->wsu__Timestamp = (_wsu__Timestamp*)soap_malloc(soap, sizeof(_wsu__Timestamp));
+  {
+    security->wsu__Timestamp = (_wsu__Timestamp*)soap_malloc(soap, sizeof(_wsu__Timestamp));
     if (!security->wsu__Timestamp)
       return soap->error = SOAP_EOM;
   }
@@ -532,20 +574,27 @@ soap_wsse_add_Timestamp(struct soap *soap, const char *id, time_t lifetime)
   return SOAP_OK;
 }
 
+/******************************************************************************/
+
 /**
 @fn _wsu__Timestamp *soap_wsse_Timestamp(struct soap *soap)
 @brief Returns Timestamp element if present.
 @param soap context
 @return _wsu__Timestamp object or NULL
 */
-struct _wsu__Timestamp*
+SOAP_FMAC1
+struct _wsu__Timestamp *
+SOAP_FMAC2
 soap_wsse_Timestamp(struct soap *soap)
-{ _wsse__Security *security = soap_wsse_Security(soap);
+{
+  _wsse__Security *security = soap_wsse_Security(soap);
   if (security)
     return security->wsu__Timestamp;
   return NULL;
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_verify_Timestamp(struct soap *soap)
 @brief Verifies the Timestamp/Expires element against the current time.
@@ -557,16 +606,21 @@ SOAP_WSSE_CLKSKEW value is used as a margin to mitigate clock skew. Keeps
 silent when no timestamp is supplied or no expiration date is included in the
 wsu:Timestamp element.
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_verify_Timestamp(struct soap *soap)
-{ _wsu__Timestamp *timestamp = soap_wsse_Timestamp(soap);
+{
+  _wsu__Timestamp *timestamp = soap_wsse_Timestamp(soap);
   DBGFUN("soap_wsse_verify_Timestamp");
   /* if we have a timestamp with an expiration date, check it */
   if (timestamp && timestamp->Expires)
-  { time_t now = time(NULL), expired;
+  {
+    time_t now = time(NULL), expired;
     soap_s2dateTime(soap, timestamp->Expires, &expired);
     if (expired + SOAP_WSSE_CLKSKEW <= now)
-    { const char *code = soap_wsu__tTimestampFault2s(soap, wsu__MessageExpired);
+    {
+      const char *code = soap_wsu__tTimestampFault2s(soap, wsu__MessageExpired);
       return soap_wsse_sender_fault_subcode(soap, code, "Message has expired", timestamp->Expires);
     }
   }
@@ -575,7 +629,7 @@ soap_wsse_verify_Timestamp(struct soap *soap)
 
 /******************************************************************************\
  *
- * wsse:Security/UsernameToken header element
+ *      wsse:Security/UsernameToken header element
  *
 \******************************************************************************/
 
@@ -593,13 +647,18 @@ Passwords are sent in the clear, so transport-level encryption is required.
 @note
 This release supports the use of at most one UsernameToken in the header.
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_add_UsernameTokenText(struct soap *soap, const char *id, const char *username, const char *password)
-{ _wsse__Security *security = soap_wsse_add_Security(soap);
+{
+  _wsse__Security *security = soap_wsse_add_Security(soap);
   DBGFUN2("soap_wsse_add_UsernameTokenText", "id=%s", id?id:"", "username=%s", username?username:"");
   /* allocate a UsernameToken if we don't have one already */
   if (!security->UsernameToken)
-  { if (!(security->UsernameToken = (_wsse__UsernameToken*)soap_malloc(soap, sizeof(_wsse__UsernameToken))))
+  {
+    security->UsernameToken = (_wsse__UsernameToken*)soap_malloc(soap, sizeof(_wsse__UsernameToken));
+    if (!security->UsernameToken)
       return soap->error = SOAP_EOM;
   }
   soap_default__wsse__UsernameToken(soap, security->UsernameToken);
@@ -608,7 +667,9 @@ soap_wsse_add_UsernameTokenText(struct soap *soap, const char *id, const char *u
   security->UsernameToken->Username = soap_strdup(soap, username);
   /* allocate and populate the Password */
   if (password)
-  { if (!(security->UsernameToken->Password = (_wsse__Password*)soap_malloc(soap, sizeof(_wsse__Password))))
+  {
+    security->UsernameToken->Password = (_wsse__Password*)soap_malloc(soap, sizeof(_wsse__Password));
+    if (!security->UsernameToken->Password)
       return soap->error = SOAP_EOM;
     soap_default__wsse__Password(soap, security->UsernameToken->Password);
     security->UsernameToken->Password->Type = (char*)wsse_PasswordTextURI;
@@ -617,6 +678,8 @@ soap_wsse_add_UsernameTokenText(struct soap *soap, const char *id, const char *u
   return SOAP_OK;
 }
 
+/******************************************************************************/
+
 /**
 @fn _wsse__UsernameToken* soap_wsse_UsernameToken(struct soap *soap, const char *id)
 @brief Returns UsernameToken element if present.
@@ -627,9 +690,12 @@ soap_wsse_add_UsernameTokenText(struct soap *soap, const char *id, const char *u
 @note
 This release supports the use of at most one UsernameToken in the header.
 */
-struct _wsse__UsernameToken*
+SOAP_FMAC1
+struct _wsse__UsernameToken *
+SOAP_FMAC2
 soap_wsse_UsernameToken(struct soap *soap, const char *id)
-{ _wsse__Security *security = soap_wsse_Security(soap);
+{
+  _wsse__Security *security = soap_wsse_Security(soap);
   if (security
    && security->UsernameToken
    && (!id || (security->UsernameToken->wsu__Id
@@ -638,6 +704,8 @@ soap_wsse_UsernameToken(struct soap *soap, const char *id)
   return NULL;
 }
 
+/******************************************************************************/
+
 /**
 @fn const char* soap_wsse_get_Username(struct soap *soap)
 @brief Returns UsernameToken/username string or wsse:FailedAuthentication fault.
@@ -649,9 +717,12 @@ The returned username should be used to lookup the user's password in a
 dictionary or database for server-side authentication with
 soap_wsse_verify_Password.
 */
-const char*
+SOAP_FMAC1
+const char *
+SOAP_FMAC2
 soap_wsse_get_Username(struct soap *soap)
-{ _wsse__UsernameToken *token = soap_wsse_UsernameToken(soap, NULL);
+{
+  _wsse__UsernameToken *token = soap_wsse_UsernameToken(soap, NULL);
   DBGFUN("soap_wsse_get_Username");
   if (token)
     return token->Username;
@@ -659,6 +730,8 @@ soap_wsse_get_Username(struct soap *soap)
   return NULL;
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_verify_Password(struct soap *soap, const char *password)
 @brief Verifies the supplied password or sets wsse:FailedAuthentication fault.
@@ -671,16 +744,21 @@ The verification supports both clear-text password verification only.
 @note
 This release supports the use of at most one UsernameToken in the header.
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_verify_Password(struct soap *soap, const char *password)
-{ _wsse__UsernameToken *token = soap_wsse_UsernameToken(soap, NULL);
+{
+  _wsse__UsernameToken *token = soap_wsse_UsernameToken(soap, NULL);
   DBGFUN("soap_wsse_verify_Password");
   /* if we have a UsernameToken with a Password, check it */
   if (token && token->Password)
-  { /* password digest or text? */
+  {
+    /* password digest or text? */
     if (token->Password->Type
      && !strcmp(token->Password->Type, wsse_PasswordTextURI))
-    { /* check password text */
+    {
+      /* check password text */
       if (!strcmp(token->Password->__item, password))
         return SOAP_OK;
     }
@@ -690,7 +768,7 @@ soap_wsse_verify_Password(struct soap *soap, const char *password)
 
 /******************************************************************************\
  *
- * Faults
+ *      Faults
  *
 \******************************************************************************/
 
@@ -703,7 +781,9 @@ soap_wsse_verify_Password(struct soap *soap, const char *password)
 @param[in] faultdetail detail string
 @return SOAP_FAULT
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_sender_fault_subcode(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail)
 {
 #if defined(SOAP_WSA_2003) || defined(SOAP_WSA_2004) || defined(SOAP_WSA_200408) || defined(SOAP_WSA_2005)
@@ -713,6 +793,8 @@ soap_wsse_sender_fault_subcode(struct soap *soap, const char *faultsubcode, cons
 #endif
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_receiver_fault_subcode(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail)
 @brief Sets receiver SOAP Fault (sub)code for server fault response.
@@ -722,7 +804,9 @@ soap_wsse_sender_fault_subcode(struct soap *soap, const char *faultsubcode, cons
 @param[in] faultdetail detail string
 @return SOAP_FAULT
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_receiver_fault_subcode(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail)
 {
 #if defined(SOAP_WSA_2003) || defined(SOAP_WSA_2004) || defined(SOAP_WSA_200408) || defined(SOAP_WSA_2005)
@@ -732,6 +816,8 @@ soap_wsse_receiver_fault_subcode(struct soap *soap, const char *faultsubcode, co
 #endif
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_sender_fault(struct soap *soap, const char *faultstring, const char *faultdetail)
 @brief Sets sender SOAP Fault for server fault response.
@@ -740,11 +826,16 @@ soap_wsse_receiver_fault_subcode(struct soap *soap, const char *faultsubcode, co
 @param[in] faultdetail detail string
 @return SOAP_FAULT
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_sender_fault(struct soap *soap, const char *faultstring, const char *faultdetail)
-{ return soap_wsse_sender_fault_subcode(soap, NULL, faultstring, faultdetail);
+{
+  return soap_wsse_sender_fault_subcode(soap, NULL, faultstring, faultdetail);
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_receiver_fault(struct soap *soap, const char *faultstring, const char *faultdetail)
 @brief Sets receiver SOAP Fault for server fault response.
@@ -753,11 +844,16 @@ soap_wsse_sender_fault(struct soap *soap, const char *faultstring, const char *f
 @param[in] faultdetail detail string
 @return SOAP_FAULT
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_receiver_fault(struct soap *soap, const char *faultstring, const char *faultdetail)
-{ return soap_wsse_receiver_fault_subcode(soap, NULL, faultstring, faultdetail);
+{
+  return soap_wsse_receiver_fault_subcode(soap, NULL, faultstring, faultdetail);
 }
 
+/******************************************************************************/
+
 /**
 @fn int soap_wsse_fault(struct soap *soap, wsse__FaultcodeEnum fault, const char *detail)
 @brief Sets SOAP Fault (sub)code for server response.
@@ -766,9 +862,12 @@ soap_wsse_receiver_fault(struct soap *soap, const char *faultstring, const char
 @param[in] detail string with optional text message
 @return SOAP_FAULT
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_fault(struct soap *soap, wsse__FaultcodeEnum fault, const char *detail)
-{ const char *code = soap_wsse__FaultcodeEnum2s(soap, fault);
+{
+  const char *code = soap_wsse__FaultcodeEnum2s(soap, fault);
   DBGFUN2("soap_wsse_fault", "fault=%s", code?code:"", "detail=%s", detail?detail:"");
   /* remove incorrect or incomplete Security header */
   soap_wsse_delete_Security(soap);
@@ -776,7 +875,8 @@ soap_wsse_fault(struct soap *soap, wsse__FaultcodeEnum fault, const char *detail
   /* detail = NULL; */ /* uncomment when detail text not recommended */
   /* use WSA to populate the SOAP Header when WSA is used */
   switch (fault)
-  { case wsse__UnsupportedSecurityToken:
+  {
+    case wsse__UnsupportedSecurityToken:
       return soap_wsse_sender_fault_subcode(soap, code, "An unsupported token was provided", detail);
     case wsse__UnsupportedAlgorithm:
       return soap_wsse_sender_fault_subcode(soap, code, "An unsupported signature or encryption algorithm was used", detail);
@@ -796,7 +896,7 @@ soap_wsse_fault(struct soap *soap, wsse__FaultcodeEnum fault, const char *detail
 
 /******************************************************************************\
  *
- * Misc functions
+ *      Misc functions
  *
 \******************************************************************************/
 
@@ -809,13 +909,18 @@ replacing colons with hyphens to produce an xsd:ID value.
 @param[in] tags string of space-separated qualified and unqualified element tag names
 @return SOAP_OK
 */
+SOAP_FMAC1
 int
+SOAP_FMAC2
 soap_wsse_set_wsu_id(struct soap *soap, const char *tags)
-{ DBGFUN1("soap_wsse_set_wsu_id", "tags=%s", tags?tags:"(null)");
+{
+  DBGFUN1("soap_wsse_set_wsu_id", "tags=%s", tags?tags:"(null)");
   soap->wsuid = soap_strdup(soap, tags);
   return SOAP_OK;
 }
 
+/******************************************************************************/
+
 #ifdef __cplusplus
 }
 #endif
