@@ -1,6 +1,6 @@
 /*
-   All OpenSSL versions prior to 1.1.0 require mutex locks to be explicitly
-   setup to work safely with multi-threaded applications
+   All OpenSSL versions prior to 1.1.0 require mutex locks to be explicitly set
+   up to work safely with multi-threaded applications
 
    At the start of your program call:
 
@@ -19,7 +19,7 @@
 
 #include "threads.h" /* gsoap/plugin/threads.h */
 
-#if defined(WITH_OPENSSL)
+#if defined(WITH_OPENSSL) && OPENSSL_VERSION_NUMBER < 0x10100000L
 
 struct CRYPTO_dynlock_value
 {
